@@ -35,6 +35,11 @@ class QgsProcessingGuiRegistry;
 class QgsProcessingRecentAlgorithmLog;
 class QgsWindowManagerInterface;
 class QgsDataItemGuiProviderRegistry;
+class QgsProviderGuiRegistry;
+class QgsProjectStorageGuiRegistry;
+class QgsNumericFormatGuiRegistry;
+class QgsCodeEditorColorSchemeRegistry;
+class QgsMessageBar;
 
 /**
  * \ingroup gui
@@ -112,6 +117,18 @@ class GUI_EXPORT QgsGui : public QObject
      */
     static QgsProcessingGuiRegistry *processingGuiRegistry() SIP_KEEPREFERENCE;
 
+    /**
+     * Returns the global numeric format gui registry, used for registering the GUI widgets associated with QgsNumericFormats.
+     * \since QGIS 3.12
+     */
+    static QgsNumericFormatGuiRegistry *numericFormatGuiRegistry() SIP_KEEPREFERENCE;
+
+    /**
+     * Returns the global code editor color scheme registry, used for registering the color schemes for QgsCodeEditor widgets.
+     * \since QGIS 3.16
+     */
+    static QgsCodeEditorColorSchemeRegistry *codeEditorColorSchemeRegistry() SIP_KEEPREFERENCE;
+
     /**
      * Returns the global processing recent algorithm log, used for tracking recently used processing algorithms.
      * \since QGIS 3.4
@@ -125,6 +142,18 @@ class GUI_EXPORT QgsGui : public QObject
      */
     static QgsDataItemGuiProviderRegistry *dataItemGuiProviderRegistry() SIP_KEEPREFERENCE;
 
+    /**
+     * Returns the global GUI-related project storage registry
+     * \since QGIS 3.10
+     */
+    static QgsProjectStorageGuiRegistry *projectStorageGuiRegistry() SIP_KEEPREFERENCE;
+
+    /**
+     * Returns the registry of GUI-related components of data providers
+     * \since QGIS 3.10
+     */
+    static QgsProviderGuiRegistry *providerGuiRegistry() SIP_KEEPREFERENCE;
+
     /**
      * Register the widget to allow its position to be automatically saved and restored when open and closed.
      * Use this to avoid needing to call saveGeometry() and restoreGeometry() on your widget.
@@ -164,10 +193,54 @@ class GUI_EXPORT QgsGui : public QObject
 
     ~QgsGui();
 
+    /**
+     * Samples the color on screen at the specified global \a point (pixel).
+     *
+     * \since QGIS 3.10
+     */
+    static QColor sampleColor( QPoint point );
+
+    /**
+     * Returns the screen at the given global \a point (pixel).
+     *
+     * \since QGIS 3.10
+     */
+    static QScreen *findScreenAt( QPoint point );
+
+    /**
+     * Returns TRUE if python macros are currently allowed to be run
+     * If the global option is to ask user, a modal dialog will be shown
+     * \param lambda a pointer to a lambda method. If specified, the dialog is not modal,
+     * a message is shown with a button to enable macro.
+     * The lambda will be run either if macros are currently allowed or if the user accepts the message.
+     * The \a messageBar must be given in such case.
+     * \param messageBar the message bar must be provided if a lambda method is used.
+     */
+    static bool pythonMacroAllowed( void ( *lambda )() = nullptr, QgsMessageBar *messageBar = nullptr ) SIP_SKIP;
+
+    ///@cond PRIVATE
+    void emitOptionsChanged() SIP_SKIP;
+    ///@endcond
+
+  signals:
+
+    /**
+     * This signal is emitted whenever the application options have been changed.
+     *
+     * This signal is a "blanket" signal, and will be emitted whenever the options dialog
+     * has been accepted regardless of whether or not individual settings are changed.
+     * It is designed as a "last resort" fallback only, allowing widgets to respond
+     * to possible settings changes.
+     *
+     * \since QGIS 3.16
+     */
+    void optionsChanged();
+
   private:
 
     QgsGui();
 
+    QgsProviderGuiRegistry *mProviderGuiRegistry = nullptr;
     QgsWidgetStateHelper *mWidgetStateHelper = nullptr;
     QgsNative *mNative = nullptr;
     QgsEditorWidgetRegistry *mEditorWidgetRegistry = nullptr;
@@ -178,7 +251,10 @@ class GUI_EXPORT QgsGui : public QObject
     QgsLayoutItemGuiRegistry *mLayoutItemGuiRegistry = nullptr;
     QgsProcessingGuiRegistry *mProcessingGuiRegistry = nullptr;
     QgsProcessingRecentAlgorithmLog *mProcessingRecentAlgorithmLog = nullptr;
+    QgsNumericFormatGuiRegistry *mNumericFormatGuiRegistry = nullptr;
     QgsDataItemGuiProviderRegistry *mDataItemGuiProviderRegistry = nullptr;
+    QgsCodeEditorColorSchemeRegistry *mCodeEditorColorSchemeRegistry = nullptr;
+    QgsProjectStorageGuiRegistry *mProjectStorageGuiRegistry = nullptr;
     std::unique_ptr< QgsWindowManagerInterface > mWindowManager;
 
 #ifdef SIP_RUN
