@@ -252,6 +252,8 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
       UndoLegendColumnCount, //!< Legend column count
       UndoLegendSymbolWidth, //!< Legend symbol width
       UndoLegendSymbolHeight, //!< Legend symbol height
+      UndoLegendMaxSymbolSize, //!< Legend maximum symbol size
+      UndoLegendMinSymbolSize, //!< Legend minimum symbol size
       UndoLegendWmsLegendWidth, //!< Legend WMS width
       UndoLegendWmsLegendHeight, //!< Legend WMS height
       UndoLegendTitleSpaceBottom, //!< Legend title space
@@ -299,6 +301,7 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
     enum Flag
     {
       FlagOverridesPaint = 1 << 1,  //!< Item overrides the default layout item painting method
+      FlagProvidesClipPath = 1 << 2, //!< Item can act as a clipping path provider (see clipPath())
     };
     Q_DECLARE_FLAGS( Flags, Flag )
 
@@ -967,6 +970,16 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
      */
     virtual bool accept( QgsStyleEntityVisitorInterface *visitor ) const;
 
+    /**
+     * Returns the clipping path generated by this item, in layout coordinates.
+     *
+     * \note Not all items can function as clipping paths. The FlagProvidesClipPath flag
+     * indicates if a particular item can function as a clipping path provider.
+     *
+     * \since QGIS 3.16
+     */
+    virtual QgsGeometry clipPath() const;
+
   public slots:
 
     /**
@@ -1042,6 +1055,15 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
      */
     void backgroundTaskCountChanged( int count );
 
+    /**
+     * Emitted when the item's clipping path has changed.
+     *
+     * \see clipPath()
+     *
+     * \since QGIS 3.16
+     */
+    void clipPathChanged();
+
   protected:
 
     /**
@@ -1059,13 +1081,26 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
      */
     virtual void draw( QgsLayoutItemRenderContext &context ) = 0;
 
+    /**
+     * Returns the path to use when drawing the item's frame or background.
+     *
+     * \see drawFrame()
+     * \see drawBackground()
+     * \since QGIS 3.16
+     */
+    virtual QPainterPath framePath() const;
+
     /**
      * Draws the frame around the item.
+     *
+     * \see framePath()
      */
     virtual void drawFrame( QgsRenderContext &context );
 
     /**
      * Draws the background for the item.
+     *
+     * \see framePath()
      */
     virtual void drawBackground( QgsRenderContext &context );
 
