@@ -24,8 +24,289 @@
 #include "qgsmaprenderercustompainterjob.h"
 #include "qgslayoutitemmapgrid.h"
 #include "qgslayoutitemmapoverview.h"
+#include "qgsmaprendererstagedrenderjob.h"
+#include "qgstemporalrangeobject.h"
 
 class QgsAnnotation;
+class QgsRenderedFeatureHandlerInterface;
+
+/**
+ * \ingroup core
+ * \class QgsLayoutItemMapAtlasClippingSettings
+ * \brief Contains settings relating to clipping a layout map by the current atlas feature.
+ * \since QGIS 3.16
+ */
+class CORE_EXPORT QgsLayoutItemMapAtlasClippingSettings : public QObject
+{
+    Q_OBJECT
+
+  public:
+
+    /**
+     * Constructor for QgsLayoutItemMapAtlasClippingSettings, with the specified \a map parent.
+     */
+    QgsLayoutItemMapAtlasClippingSettings( QgsLayoutItemMap *map SIP_TRANSFERTHIS = nullptr );
+
+    /**
+     * Returns TRUE if the map content should be clipped to the current atlas feature.
+     *
+     * \see setEnabled()
+     */
+    bool enabled() const;
+
+    /**
+     * Sets whether the map content should be clipped to the current atlas feature.
+     *
+     * \see enabled()
+     */
+    void setEnabled( bool enabled );
+
+    /**
+     * Returns the feature clipping type to apply when clipping to the current atlas feature.
+     *
+     * \see setFeatureClippingType()
+     */
+    QgsMapClippingRegion::FeatureClippingType featureClippingType() const;
+
+    /**
+     * Sets the feature clipping \a type to apply when clipping to the current atlas feature.
+     *
+     * \see featureClippingType()
+     */
+    void setFeatureClippingType( QgsMapClippingRegion::FeatureClippingType type );
+
+    /**
+     * Returns TRUE if labels should only be placed inside the atlas feature geometry.
+     *
+     * \see setForceLabelsInsideFeature()
+     */
+    bool forceLabelsInsideFeature() const;
+
+    /**
+     * Sets whether labels should only be placed inside the atlas feature geometry.
+     *
+     * \see forceLabelsInsideFeature()
+     */
+    void setForceLabelsInsideFeature( bool forceInside );
+
+    /**
+     * Returns TRUE if clipping should be restricted to a subset of layers.
+     *
+     * \see layersToClip()
+     * \see setRestrictToLayers()
+     */
+    bool restrictToLayers() const;
+
+    /**
+     * Sets whether clipping should be restricted to a subset of layers.
+     *
+     * \see setLayersToClip()
+     * \see restrictToLayers()
+     */
+    void setRestrictToLayers( bool enabled );
+
+    /**
+     * Returns the list of map layers to clip to the atlas feature.
+     *
+     * \note This setting is only used if restrictToLayers() is TRUE.
+     *
+     * \see restrictToLayers()
+     * \see setLayersToClip()
+     */
+    QList< QgsMapLayer * > layersToClip() const;
+
+    /**
+     * Sets the list of map \a layers to clip to the atlas feature.
+     *
+     * \note This setting is only used if restrictToLayers() is TRUE.
+     *
+     * \see restrictToLayers()
+     * \see layersToClip()
+     */
+    void setLayersToClip( const QList< QgsMapLayer * > &layers );
+
+    /**
+     * Stores settings in a DOM element, where \a element is the DOM element
+     * corresponding to a 'LayoutMap' tag.
+     * \see readXml()
+     */
+    bool writeXml( QDomElement &element, QDomDocument &document, const QgsReadWriteContext &context ) const;
+
+    /**
+     * Sets the setting's state from a DOM document, where \a element is the DOM
+     * node corresponding to a 'LayoutMap' tag.
+     * \see writeXml()
+     */
+    bool readXml( const QDomElement &element, const QDomDocument &doc, const QgsReadWriteContext &context );
+
+  signals:
+
+    /**
+     * Emitted when the atlas clipping settings are changed.
+     */
+    void changed();
+
+  private slots:
+    void layersAboutToBeRemoved( const QList<QgsMapLayer *> &layers );
+
+  private:
+
+    QgsLayoutItemMap *mMap = nullptr;
+    bool mClipToAtlasFeature = false;
+    bool mRestrictToLayers = false;
+    QList< QgsMapLayerRef > mLayersToClip;
+    QgsMapClippingRegion::FeatureClippingType mFeatureClippingType = QgsMapClippingRegion::FeatureClippingType::ClipPainterOnly;
+    bool mForceLabelsInsideFeature = false;
+};
+
+
+/**
+ * \ingroup core
+ * \class QgsLayoutItemMapItemClipPathSettings
+ * \brief Contains settings relating to clipping a layout map by another layout item.
+ * \since QGIS 3.16
+ */
+class CORE_EXPORT QgsLayoutItemMapItemClipPathSettings : public QObject
+{
+    Q_OBJECT
+
+  public:
+
+    /**
+     * Constructor for QgsLayoutItemMapItemClipPathSettings, with the specified \a map parent.
+     */
+    QgsLayoutItemMapItemClipPathSettings( QgsLayoutItemMap *map SIP_TRANSFERTHIS = nullptr );
+
+    /**
+     * Returns TRUE if the item clipping is enabled and set to a valid source item.
+     *
+     * \see enabled()
+     * \see sourceItem()
+     */
+    bool isActive() const;
+
+    /**
+     * Returns TRUE if the map content should be clipped to the associated item.
+     *
+     * \see setEnabled()
+     */
+    bool enabled() const;
+
+    /**
+     * Sets whether the map content should be clipped to the associated item.
+     *
+     * \see enabled()
+     */
+    void setEnabled( bool enabled );
+
+    /**
+     * Returns the geometry to use for clipping the parent map, in the map item's CRS.
+     *
+     * \see clipPathInMapItemCoordinates()
+     */
+    QgsGeometry clippedMapExtent() const;
+
+    /**
+     * Returns the clipping path geometry, in the map item's coordinate space.
+     *
+     * \warning The return path is not in geographic coordinates, rather the map
+     * layout item's QGraphicsItem coordinate space. Use clippedMapExtent() to retrieve
+     * the clip path in the map's CRS.
+     *
+     * \see clippedMapExtent()
+     */
+    QgsGeometry clipPathInMapItemCoordinates() const;
+
+    /**
+     * Returns the clip path as a map clipping region.
+     */
+    QgsMapClippingRegion toMapClippingRegion() const;
+
+    /**
+     * Sets the source \a item which will provide the clipping path for the map.
+     *
+     * The specified \a item must return the QgsLayoutItem::FlagProvidesClipPath flag.
+     *
+     * \see sourceItem()
+     */
+    void setSourceItem( QgsLayoutItem *item );
+
+    /**
+     * Returns the source item which will provide the clipping path for the map, or NULLPTR
+     * if no item is set.
+     *
+     * \see setSourceItem()
+     */
+    QgsLayoutItem *sourceItem();
+
+    /**
+     * Returns the feature clipping type to apply when clipping to the associated item.
+     *
+     * \see setFeatureClippingType()
+     */
+    QgsMapClippingRegion::FeatureClippingType featureClippingType() const;
+
+    /**
+     * Sets the feature clipping \a type to apply when clipping to the associated item.
+     *
+     * \see featureClippingType()
+     */
+    void setFeatureClippingType( QgsMapClippingRegion::FeatureClippingType type );
+
+    /**
+     * Returns TRUE if labels should only be placed inside the clip path geometry.
+     *
+     * \see setForceLabelsInsideClipPath()
+     */
+    bool forceLabelsInsideClipPath() const;
+
+    /**
+     * Sets whether labels should only be placed inside the clip path geometry.
+     *
+     * \see forceLabelsInsideClipPath()
+     */
+    void setForceLabelsInsideClipPath( bool forceInside );
+
+    /**
+     * Stores settings in a DOM element, where \a element is the DOM element
+     * corresponding to a 'LayoutMap' tag.
+     * \see readXml()
+     */
+    bool writeXml( QDomElement &element, QDomDocument &document, const QgsReadWriteContext &context ) const;
+
+    /**
+     * Sets the setting's state from a DOM document, where \a element is the DOM
+     * node corresponding to a 'LayoutMap' tag.
+     * \see writeXml()
+     * \see finalizeRestoreFromXml()
+     */
+    bool readXml( const QDomElement &element, const QDomDocument &doc, const QgsReadWriteContext &context );
+
+    /**
+     * To be called after all pending items have been restored from XML.
+     * \see readXml()
+     */
+    void finalizeRestoreFromXml();
+
+  signals:
+
+    /**
+     * Emitted when the item clipping settings are changed.
+     */
+    void changed();
+
+  private:
+
+    QgsLayoutItemMap *mMap = nullptr;
+    bool mEnabled = false;
+    QgsMapClippingRegion::FeatureClippingType mFeatureClippingType = QgsMapClippingRegion::FeatureClippingType::ClipPainterOnly;
+    bool mForceLabelsInsideClipPath = false;
+
+    QPointer< QgsLayoutItem > mClipPathSource;
+    QString mClipPathUuid;
+
+};
+
 
 /**
  * \ingroup core
@@ -33,7 +314,7 @@ class QgsAnnotation;
  * \brief Layout graphical items for displaying a map.
  * \since QGIS 3.0
  */
-class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
+class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem, public QgsTemporalRangeObject
 {
 
     Q_OBJECT
@@ -69,7 +350,8 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
      */
     enum MapItemFlag
     {
-      ShowPartialLabels  = 1 << 0,  //!< Whether to draw labels which are partially outside of the map view
+      ShowPartialLabels  = 1 << 0, //!< Whether to draw labels which are partially outside of the map view
+      ShowUnplacedLabels = 1 << 1, //!< Whether to render unplaced labels in the map view
     };
     Q_DECLARE_FLAGS( MapItemFlags, MapItemFlag )
 
@@ -114,7 +396,12 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
 
     // for now, map items behave a bit differently and don't implement draw. TODO - see if we can avoid this
     void paint( QPainter *painter, const QStyleOptionGraphicsItem *itemStyle, QWidget *pWidget ) override;
-    int numberExportLayers() const override;
+    Q_DECL_DEPRECATED int numberExportLayers() const override SIP_DEPRECATED;
+    void startLayeredExport() override;
+    void stopLayeredExport() override;
+    bool nextExportPart() override;
+    ExportLayerBehavior exportLayerBehavior() const override;
+    QgsLayoutItem::ExportLayerDetail exportLayerDetails() const override;
     void setFrameStrokeWidth( QgsLayoutMeasurement width ) override;
 
     /**
@@ -263,7 +550,7 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
     void storeCurrentLayerStyles();
 
     /**
-     * Returns whether the map should follow a map theme. If true, the layers and layer styles
+     * Returns whether the map should follow a map theme. If TRUE, the layers and layer styles
      * will be used from given preset name (configured with setFollowVisibilityPresetName() method).
      * This means when preset's settings are changed, the new settings are automatically
      * picked up next time when rendering, without having to explicitly update them.
@@ -277,20 +564,24 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
     /**
      * Sets whether the map should follow a map theme. See followVisibilityPreset() for more details.
      */
-    void setFollowVisibilityPreset( bool follow ) { mFollowVisibilityPreset = follow; }
+    void setFollowVisibilityPreset( bool follow );
 
     /**
      * Preset name that decides which layers and layer styles are used for map rendering. It is only
-     * used when followVisibilityPreset() returns true.
+     * used when followVisibilityPreset() returns TRUE.
      * \see setFollowVisibilityPresetName()
+     *
+     * \see themeChanged()
      */
     QString followVisibilityPresetName() const { return mFollowVisibilityPresetName; }
 
     /**
      * Sets preset name for map rendering. See followVisibilityPresetName() for more details.
      * \see followVisibilityPresetName()
+     *
+     * \see themeChanged()
     */
-    void setFollowVisibilityPresetName( const QString &name ) { mFollowVisibilityPresetName = name; }
+    void setFollowVisibilityPresetName( const QString &name );
 
     void moveContent( double dx, double dy ) override;
     void setMoveContentPreviewOffset( double dx, double dy ) override;
@@ -298,7 +589,7 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
     void zoomContent( double factor, QPointF point ) override;
 
 
-    //! Returns true if the map contains a WMS layer.
+    //! Returns TRUE if the map contains a WMS layer.
     bool containsWmsLayer() const;
 
     bool requiresRasterization() const override;
@@ -337,7 +628,7 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
 
     /**
      * Returns whether the map extent is set to follow the current atlas feature.
-     * \returns true if map will follow the current atlas feature.
+     * \returns TRUE if map will follow the current atlas feature.
      * \see setAtlasDriven
      * \see atlasScalingMode
      */
@@ -345,7 +636,7 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
 
     /**
      * Sets whether the map extent will follow the current atlas feature.
-     * \param enabled set to true if the map extents should be set by the current atlas feature.
+     * \param enabled set to TRUE if the map extents should be set by the current atlas feature.
      * \see atlasDriven
      * \see setAtlasScalingMode
      */
@@ -356,7 +647,7 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
      * are calculated for the current atlas feature when an atlas composition
      * is enabled.
      * \returns the current scaling mode
-     * \note this parameter is only used if atlasDriven() is true
+     * \note this parameter is only used if atlasDriven() is TRUE
      * \see setAtlasScalingMode
      * \see atlasDriven
      */
@@ -367,7 +658,7 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
      * are calculated for the current atlas feature when an atlas composition
      * is enabled.
      * \param mode atlas scaling mode to set
-     * \note this parameter is only used if atlasDriven() is true
+     * \note this parameter is only used if atlasDriven() is TRUE
      * \see atlasScalingMode
      * \see atlasDriven
      */
@@ -458,9 +749,9 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
     /**
      * Returns map settings that will be used for drawing of the map.
      *
-     * If \a includeLayerSettings is true, than settings specifically relating to map layers and map layer styles
+     * If \a includeLayerSettings is TRUE, than settings specifically relating to map layers and map layer styles
      * will be calculated. This can be expensive to calculate, so if they are not required in the map settings
-     * (e.g. for map settings which are used for scale related calculations only) then \a includeLayerSettings should be false.
+     * (e.g. for map settings which are used for scale related calculations only) then \a includeLayerSettings should be FALSE.
      */
     QgsMapSettings mapSettings( const QgsRectangle &extent, QSizeF size, double dpi, bool includeLayerSettings ) const;
 
@@ -499,7 +790,7 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
     void removeLabelBlockingItem( QgsLayoutItem *item );
 
     /**
-     * Returns true if the specified \a item is a "label blocking item".
+     * Returns TRUE if the specified \a item is a "label blocking item".
      *
      * Items which are marked as label blocking items prevent any map labels from being placed
      * in the area of the map item covered by the item.
@@ -517,11 +808,55 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
      */
     QgsMapRendererJob::Errors renderingErrors() const { return mRenderingErrors; }
 
+    bool accept( QgsStyleEntityVisitorInterface *visitor ) const override;
+
+    /**
+     * Adds a rendered feature \a handler to use while rendering the map.
+     *
+     * Ownership of \a handler is NOT transferred, and it is the caller's responsibility to ensure
+     * that the handler exists for as long as it is registered with the map item.
+     *
+     * Callers should call removeRenderedFeatureHandler() to remove the handler before
+     * destroying the \a handler.
+     *
+     * \see removeRenderedFeatureHandler()
+     * \since QGIS 3.10
+     */
+    void addRenderedFeatureHandler( QgsRenderedFeatureHandlerInterface *handler );
+
+    /**
+     * Removes a previously added rendered feature \a handler.
+     *
+     * \see addRenderedFeatureHandler()
+     * \since QGIS 3.10
+     */
+    void removeRenderedFeatureHandler( QgsRenderedFeatureHandlerInterface *handler );
+
+    /**
+     * Creates a transform from layout coordinates to map coordinates.
+     */
+    QTransform layoutToMapCoordsTransform() const;
+
+    /**
+     * Returns the map's atlas clipping settings.
+     *
+     * \since QGIS 3.16
+     */
+    QgsLayoutItemMapAtlasClippingSettings *atlasClippingSettings() { return mAtlasClippingSettings; }
+
+    /**
+     * Returns the map's item based clip path settings.
+     *
+     * \since QGIS 3.16
+     */
+    QgsLayoutItemMapItemClipPathSettings *itemClippingSettings() { return mItemClippingSettings; }
+
   protected:
 
     void draw( QgsLayoutItemRenderContext &context ) override;
     bool writePropertiesToElement( QDomElement &element, QDomDocument &document, const QgsReadWriteContext &context ) const override;
     bool readPropertiesFromElement( const QDomElement &element, const QDomDocument &document, const QgsReadWriteContext &context ) override;
+    QPainterPath framePath() const override;
 
     //! True if a draw is already in progress
     bool isDrawing() const {return mDrawing;}
@@ -543,20 +878,20 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
   signals:
 
     /**
-     * Is emitted when the map's extent changes.
+     * Emitted when the map's extent changes.
      * \see setExtent()
      * \see extent()
      */
     void extentChanged();
 
     /**
-     * Is emitted when the map's rotation changes.
+     * Emitted when the map's rotation changes.
      * \see setMapRotation()
      * \see mapRotation()
      */
     void mapRotationChanged( double newRotation );
 
-    //! Is emitted when the map has been prepared for atlas rendering, just before actual rendering
+    //! Emitted when the map has been prepared for atlas rendering, just before actual rendering
     void preparedForAtlas();
 
     /**
@@ -565,6 +900,23 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
      */
     void layerStyleOverridesChanged();
 
+    /**
+     * Emitted when the map's associated \a theme is changed.
+     *
+     * \note This signal is not emitted when the definition of the theme changes, only the map
+     * is linked to a different theme then it previously was.
+     *
+     * \since QGIS 3.14
+     */
+    void themeChanged( const QString &theme );
+
+    /**
+     * Emitted when the map's coordinate reference system is changed.
+     *
+     * \since QGIS 3.18
+     */
+    void crsChanged();
+
   public slots:
 
     void refresh() override;
@@ -585,12 +937,16 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
 
     void mapThemeChanged( const QString &theme );
 
+    //! Renames the active map theme called \a theme to \a newTheme
+    void currentMapThemeRenamed( const QString &theme, const QString &newTheme );
+
     //! Create cache image
     void recreateCachedImageInBackground();
 
+    void updateAtlasFeature();
   private:
 
-    QgsLayoutItemMap::MapItemFlags mMapFlags = nullptr;
+    QgsLayoutItemMap::MapItemFlags mMapFlags = QgsLayoutItemMap::MapItemFlags();
 
     //! Unique identifier
     int mMapId = 1;
@@ -649,10 +1005,11 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
 
     /**
      * Temporary evaluated map rotation. Data defined rotation may mean this value
-     * differs from mMapRotation*/
+     * differs from mMapRotation
+    */
     double mEvaluatedMapRotation = 0;
 
-    //! Flag if layers to be displayed should be read from qgis canvas (true) or from stored list in mLayerSet (false)
+    //! Flag if layers to be displayed should be read from qgis canvas (TRUE) or from stored list in mLayerSet (FALSE)
     bool mKeepLayerSet = false;
 
     //! Stored layer list (used if layer live-link mKeepLayerSet is disabled)
@@ -670,14 +1027,19 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
     /**
      * Whether layers and styles should be used from a preset (preset name is stored
      * in mVisibilityPresetName and may be overridden by data-defined expression).
-     * This flag has higher priority than mKeepLayerSet. */
+     * This flag has higher priority than mKeepLayerSet.
+    */
     bool mFollowVisibilityPreset = false;
 
     /**
      * Map theme name to be used for map's layers and styles in case mFollowVisibilityPreset
-     *  is true. May be overridden by data-defined expression. */
+     * is TRUE. May be overridden by data-defined expression.
+    */
     QString mFollowVisibilityPresetName;
 
+    //! Name of the last data-defined evaluated theme name
+    QString mLastEvaluatedThemeName;
+
     /**
      * \brief Draw to paint device
      *  \param painter painter
@@ -699,11 +1061,6 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
     //! Returns first map overview or creates an empty one if none
     const QgsLayoutItemMapOverview *constFirstMapOverview() const;
 
-    /**
-     * Creates a transform from layout coordinates to map coordinates.
-     */
-    QTransform layoutToMapCoordsTransform() const;
-
     /**
      * Creates a list of label blocking regions for the map, which correspond to the
      * map areas covered by other layout items marked as label blockers for this map.
@@ -735,11 +1092,17 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
     //!layer id / error message
     QgsMapRendererJob::Errors mRenderingErrors;
 
+    QList< QgsRenderedFeatureHandlerInterface * > mRenderedFeatureHandlers;
+
+    std::unique_ptr< QgsMapRendererStagedRenderJob > mStagedRendererJob;
+
     void init();
 
     //! Resets the item tooltip to reflect current map id
     void updateToolTip();
 
+    QString themeToRender( const QgsExpressionContext &context ) const;
+
     //! Returns current layer style overrides for this map item
     QMap<QString, QString> layerStyleOverridesToRender( const QgsExpressionContext &context ) const;
 
@@ -765,17 +1128,27 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
 
     enum PartType
     {
+      Start,
       Background,
       Layer,
       Grid,
       OverviewMapExtent,
       Frame,
-      SelectionBoxes
+      SelectionBoxes,
+      End,
+      NotLayered,
     };
 
     //! Test if a part of the item needs to be drawn, considering the context's current export layer
     bool shouldDrawPart( PartType part ) const;
 
+    PartType mCurrentExportPart = NotLayered;
+    QStringList mExportThemes;
+    QStringList::iterator mExportThemeIt;
+
+    QgsLayoutItemMapAtlasClippingSettings *mAtlasClippingSettings = nullptr;
+    QgsLayoutItemMapItemClipPathSettings *mItemClippingSettings = nullptr;
+
     /**
      * Refresh the map's extents, considering data defined extent, scale and rotation
      * \param context expression context for evaluating data defined map parameters
@@ -784,15 +1157,18 @@ class CORE_EXPORT QgsLayoutItemMap : public QgsLayoutItem
 
     void refreshLabelMargin( bool updateItem );
 
-    void updateAtlasFeature();
-
     QgsRectangle computeAtlasRectangle();
 
+    void createStagedRenderJob( const QgsRectangle &extent, const QSizeF size, double dpi );
+
+    QPolygonF calculateVisibleExtentPolygon( bool includeClipping ) const;
+
     friend class QgsLayoutItemMapGrid;
     friend class QgsLayoutItemMapOverview;
     friend class QgsLayoutItemLegend;
     friend class TestQgsLayoutMap;
     friend class QgsCompositionConverter;
+    friend class QgsGeoPdfRenderedFeatureHandler;
 
 };
 
