@@ -17,16 +17,12 @@
 ***************************************************************************
 """
 
-
 __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = 'bc43194061452b37c8a22edcb7b380055a0c4236'
-
 import os
+import shutil
 import importlib
 from qgis.core import (Qgis,
                        QgsApplication,
@@ -49,10 +45,10 @@ from qgis.core import (Qgis,
                        QgsProcessingParameterRasterDestination,
                        QgsProcessingParameterVectorDestination)
 from processing.core.ProcessingConfig import ProcessingConfig
-from processing.core.parameters import getParameterFromString
 from processing.algs.help import shortHelp
 from processing.tools.system import getTempFilename
 from processing.algs.saga.SagaNameDecorator import decoratedAlgorithmName, decoratedGroupName
+from processing.algs.saga.SagaParameters import Parameters
 from . import SagaUtils
 from .SagaAlgorithmBase import SagaAlgorithmBase
 
@@ -63,7 +59,6 @@ sessionExportedLayers = {}
 
 
 class SagaAlgorithm(SagaAlgorithmBase):
-
     OUTPUT_EXTENT = 'OUTPUT_EXTENT'
 
     def __init__(self, descriptionfile):
@@ -77,6 +72,7 @@ class SagaAlgorithm(SagaAlgorithmBase):
         self._group = ''
         self._groupId = ''
         self.params = []
+        self.known_issues = False
         self.defineCharacteristicsFromFile()
 
     def createInstance(self):
@@ -109,11 +105,15 @@ class SagaAlgorithm(SagaAlgorithmBase):
 
     def flags(self):
         # TODO - maybe it's safe to background thread this?
-        return super().flags() | QgsProcessingAlgorithm.FlagNoThreading
+        f = super().flags() | QgsProcessingAlgorithm.FlagNoThreading
+        if self.known_issues:
+            f = f | QgsProcessingAlgorithm.FlagKnownIssues
+        return f
 
     def defineCharacteristicsFromFile(self):
         with open(self.description_file, encoding="utf-8") as lines:
             line = lines.readline().strip('\n').strip()
+
             self._name = line
             if '|' in self._name:
                 tokens = self._name.split('|')
@@ -133,6 +133,10 @@ class SagaAlgorithm(SagaAlgorithmBase):
             self._name = ''.join(c for c in self._name if c in validChars)
 
             line = lines.readline().strip('\n').strip()
+            if line == '##known_issues':
+                self.known_issues = True
+                line = lines.readline().strip('\n').strip()
+
             self.undecorated_group = line
             self._group = self.tr(decoratedGroupName(self.undecorated_group))
 
@@ -143,13 +147,13 @@ class SagaAlgorithm(SagaAlgorithmBase):
             while line != '':
                 if line.startswith('Hardcoded'):
                     self.hardcoded_strings.append(line[len('Hardcoded|'):])
-                elif line.startswith('QgsProcessingParameter') or line.startswith('Parameter'):
-                    self.params.append(getParameterFromString(line))
+                elif Parameters.is_parameter_line(line):
+                    self.params.append(Parameters.create_parameter_from_line(line))
                 elif line.startswith('AllowUnmatching'):
                     self.allow_nonmatching_grid_extents = True
                 else:
-                    pass # TODO
-                    #self.addOutput(getOutputFromString(line))
+                    pass  # TODO
+                    # self.addOutput(getOutputFromString(line))
                 line = lines.readline().strip('\n').strip()
 
     def processAlgorithm(self, parameters, context, feedback):
@@ -170,7 +174,7 @@ class SagaAlgorithm(SagaAlgorithmBase):
                 if isinstance(parameters[param.name()], str):
                     if parameters[param.name()].lower().endswith('sdat'):
                         self.exportedLayers[param.name()] = parameters[param.name()][:-4] + 'sgrd'
-                    if parameters[param.name()].lower().endswith('sgrd'):
+                    elif parameters[param.name()].lower().endswith('sgrd'):
                         self.exportedLayers[param.name()] = parameters[param.name()]
                     else:
                         layer = self.parameterAsRasterLayer(parameters, param.name(), context)
@@ -214,9 +218,9 @@ class SagaAlgorithm(SagaAlgorithmBase):
                     files = []
                     for i, layer in enumerate(layers):
                         if layer.source().lower().endswith('sdat'):
-                            files.append(parameters[param.name()].source()[:-4] + 'sgrd')
-                        if layer.source().lower().endswith('sgrd'):
-                            files.append(parameters[param.name()].source())
+                            files.append(layer.source()[:-4] + 'sgrd')
+                        elif layer.source().lower().endswith('sgrd'):
+                            files.append(layer.source())
                         else:
                             exportCommand = self.exportRasterLayer(param.name(), layer)
                             files.append(self.exportedLayers[param.name()])
@@ -260,10 +264,10 @@ class SagaAlgorithm(SagaAlgorithmBase):
             if isinstance(param, (QgsProcessingParameterRasterLayer, QgsProcessingParameterFeatureSource)):
                 command += ' -{} "{}"'.format(param.name(), self.exportedLayers[param.name()])
             elif isinstance(param, QgsProcessingParameterMultipleLayers):
-                if parameters[param.name()]: # parameter may have been an empty list
+                if parameters[param.name()]:  # parameter may have been an empty list
                     command += ' -{} "{}"'.format(param.name(), ';'.join(self.exportedLayers[param.name()]))
             elif isinstance(param, QgsProcessingParameterBoolean):
-                if self.parameterAsBool(parameters, param.name(), context):
+                if self.parameterAsBoolean(parameters, param.name(), context):
                     command += ' -{} true'.format(param.name().strip())
                 else:
                     command += ' -{} false'.format(param.name().strip())
@@ -277,7 +281,7 @@ class SagaAlgorithm(SagaAlgorithmBase):
                         f.write(s)
                 command += ' -{} "{}"'.format(param.name(), tempTableFile)
             elif isinstance(param, QgsProcessingParameterExtent):
-                # 'We have to substract/add half cell size, since SAGA is
+                # 'We have to subtract/add half cell size, since SAGA is
                 # center based, not corner based
                 halfcell = self.getOutputCellsize(parameters, context) / 2
                 offset = [halfcell, -halfcell, halfcell, -halfcell]
@@ -305,13 +309,21 @@ class SagaAlgorithm(SagaAlgorithmBase):
 
         output_layers = []
         output_files = {}
+        # If the user has entered an output file that has non-ascii chars, we use a different path with only ascii chars
+        output_files_nonascii = {}
         for out in self.destinationParameterDefinitions():
             filePath = self.parameterAsOutputLayer(parameters, out.name(), context)
             if isinstance(out, (QgsProcessingParameterRasterDestination, QgsProcessingParameterVectorDestination)):
                 output_layers.append(filePath)
+                try:
+                    filePath.encode('ascii')
+                except UnicodeEncodeError:
+                    nonAsciiFilePath = filePath
+                    filePath = QgsProcessingUtils.generateTempFilename(out.name() + os.path.splitext(filePath)[1])
+                    output_files_nonascii[filePath] = nonAsciiFilePath
+
             output_files[out.name()] = filePath
             command += ' -{} "{}"'.format(out.name(), filePath)
-
         commands.append(command)
 
         # special treatment for RGB algorithm
@@ -321,7 +333,7 @@ class SagaAlgorithm(SagaAlgorithmBase):
                 filename = self.parameterAsOutputLayer(parameters, out.name(), context)
                 filename2 = os.path.splitext(filename)[0] + '.sgrd'
                 if self.cmdname == 'RGB Composite':
-                    commands.append('io_grid_image 0 -IS_RGB -GRID:"{}" -FILE:"{}"'.format(filename2, filename))
+                    commands.append('io_grid_image 0 -COLOURING 4 -GRID:"{}" -FILE:"{}"'.format(filename2, filename))
 
         # 3: Run SAGA
         commands = self.editCommands(commands)
@@ -341,6 +353,17 @@ class SagaAlgorithm(SagaAlgorithmBase):
                 with open(prjFile, 'w') as f:
                     f.write(crs.toWkt())
 
+        for old, new in output_files_nonascii.items():
+            oldFolder = os.path.dirname(old)
+            newFolder = os.path.dirname(new)
+            newName = os.path.splitext(os.path.basename(new))[0]
+            files = [f for f in os.listdir(oldFolder)]
+            for f in files:
+                ext = os.path.splitext(f)[1]
+                newPath = os.path.join(newFolder, newName + ext)
+                oldPath = os.path.join(oldFolder, f)
+                shutil.move(oldPath, newPath)
+
         result = {}
         for o in self.outputDefinitions():
             if o.name() in output_files:
@@ -421,8 +444,8 @@ class SagaAlgorithm(SagaAlgorithmBase):
 
             if isinstance(param, QgsProcessingParameterRasterLayer):
                 raster_layer_params.append(param.name())
-            elif (isinstance(param, QgsProcessingParameterMultipleLayers) and
-                    param.layerType() == QgsProcessing.TypeRaster):
+            elif (isinstance(param, QgsProcessingParameterMultipleLayers)
+                    and param.layerType() == QgsProcessing.TypeRaster):
                 raster_layer_params.extend(param.name())
 
         for layer_param in raster_layer_params:
